/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is WebMail Notifier.
 *
 * The Initial Developer of the Original Code is
 * Byung-Wook Kang.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/***********************************************************
constants
***********************************************************/

const nsIWebMailNotifier = Components.interfaces.nsIWebMailNotifier;

// UUID uniquely identifying our component
const CLASS_ID = Components.ID("{e3ab0980-b4e1-11db-abbd-0800200c9a66}");

// description
const CLASS_NAME = "WebMail Notifier Service";

// textual unique identifier
const CONTRACT_ID = "@mozilla.org/WebMailNotifier;1";

/***********************************************************
class definition
***********************************************************/

//class constructor
function WebMailNotifier() {
  //preference observer
  var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefService);
  this.prefBranch = prefService.getBranch("wm-notifier.");
  this.prefBranch.QueryInterface(Components.interfaces.nsIPrefBranch2);
  this.prefBranch.addObserver("", this, false);
  this.timerDelay=this.prefBranch.getIntPref("updateInterval")*60000;

  this.webMailHandlers=[];
  for(var i=0;i<nsIWebMailNotifier.NUM_OF_MAILS;i++){
    this.webMailHandlers[i]=new WebMailHandler(this);
    this.webMailHandlers[i].id=i;
    this.webMailHandlers[i].enabled=this.prefBranch.getBoolPref("webmail"+i+".enabled");
  }
  initDaumHandler(this.webMailHandlers[nsIWebMailNotifier.MAIL_DAUM]);
  initNaverHandler(this.webMailHandlers[nsIWebMailNotifier.MAIL_NAVER]);
  initEmpasHandler(this.webMailHandlers[nsIWebMailNotifier.MAIL_EMPAS]);
  initNateHandler(this.webMailHandlers[nsIWebMailNotifier.MAIL_NATE]);
  initGmailHandler(this.webMailHandlers[nsIWebMailNotifier.MAIL_GMAIL]);
  initYahooHandler(this.webMailHandlers[nsIWebMailNotifier.MAIL_YAHOO]);
  initLiveHandler(this.webMailHandlers[nsIWebMailNotifier.MAIL_LIVE]);
}

// class definition
WebMailNotifier.prototype = {
  // define the function we want to expose in our interface
  listeners : new Array(),
  timer : null,
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(nsIWebMailNotifier) &&
        !aIID.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};
WebMailNotifier.prototype.addListener = function(aListener) {
  this.listeners.push(aListener);
  if (!this.timer){//first listener
    this.checkMails();
    this.setTimer();
  }else{
    for(var i=0;i<this.webMailHandlers.length;i++){
      aListener.onStateChange(i,this.webMailHandlers[i].mailCount);
    }
  }
}
WebMailNotifier.prototype.removeListener = function(aListener){
  for(var i=0;i<this.listeners.length;i++){
    if(this.listeners[i]==aListener){
        this.listeners.splice(i,1);
        break;
    }
  }
}
WebMailNotifier.prototype.checkMails = function() {
  for(var i=0;i<this.webMailHandlers.length;i++){
    this.webMailHandlers[i].checkMail();
  }
}
WebMailNotifier.prototype.getMailURL = function(aID) {
    if(this.webMailHandlers[aID]){
      return this.webMailHandlers[aID].getMailURL();
    }
    return null;
}
WebMailNotifier.prototype.setState = function(aID,aCount) {
  for(var i=0;i<this.listeners.length;i++){
    if(this.listeners[i]){
      this.listeners[i].onStateChange(aID,aCount);
    }
  }
}

/***********************************************************
timer
***********************************************************/
WebMailNotifier.prototype.setTimer = function() {
  if (this.timer) {
    this.timer.cancel();
  } else {
    this.timer = Components.classes["@mozilla.org/timer;1"]
                  .createInstance(Components.interfaces.nsITimer);
  }
  try{
    this.timer.initWithCallback(this,
                                this.timerDelay,
                                Components.interfaces.TYPE_ONE_SHOT);
  } catch (e) {
  }
}

WebMailNotifier.prototype.notify = function(aTimer) {
  this.checkMails();
  this.setTimer();
}
/***********************************************************
load html
***********************************************************/
WebMailNotifier.prototype.getHtml = function(aHandler,aURL,aPostData) {
  var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
  // create an nsIURI
  var uri = ioService.newURI(aURL, null, null);
  //nsIInputStream
  var uploadStream = Components.classes["@mozilla.org/io/string-input-stream;1"]
    .createInstance(Components.interfaces.nsIStringInputStream);
  if (aPostData)
    uploadStream.setData(aPostData, aPostData.length);

  // get a channel for that nsIURI
  var channel = ioService.newChannelFromURI(uri);

  // get a httpchannel and make it a post
  var httpChannel = channel.QueryInterface(Components.interfaces.nsIHttpChannel);
  if (aPostData) {
    var uploadChannel = channel.QueryInterface(Components.interfaces.nsIUploadChannel);
    uploadChannel.setUploadStream(uploadStream, "application/x-www-form-urlencoded", -1);
    httpChannel.requestMethod = "POST";
  }
  channel.asyncOpen(aHandler,httpChannel);
}
/***********************************************************
preference observer
***********************************************************/
WebMailNotifier.prototype.observe=function(aSubject, aTopic, aData)
{
  if(aTopic != "nsPref:changed") return;
  switch (aData) {
    case "updateInterval":
      this.timerDelay=this.prefBranch.getIntPref(aData)*60000;
      break;
    default:
      var fnd=aData.match(/webmail(\d+).enabled/);
      if(fnd){
        this.webMailHandlers[fnd[1]].enabled=this.prefBranch.getBoolPref(aData);
      }
  }
}
/***********************************************************
password manager
***********************************************************/
WebMailNotifier.prototype.getPassword=function(mailid,userid){
  var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"]
                                .getService(Components.interfaces.nsIPasswordManager);
  var e = passwordManager.enumerator;
  var str="chrome://wm-notifier/webmail"+mailid;
  while (e.hasMoreElements()) {
    try {
      var pass = e.getNext().QueryInterface(Components.interfaces.nsIPassword);
      if (pass.host == str&&pass.user==userid) {
         return pass.password; // the password
      }
    }catch(ex) {
    }
  }
  return "";
}

/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID.
***********************************************************/
var WebMailNotifierFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new WebMailNotifier()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var WebMailNotifierModule = {
  _firstTime: true,
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME,
        CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);
  },

  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return WebMailNotifierFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },
  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
function NSGetModule(aCompMgr, aFileSpec) { return WebMailNotifierModule; }


/***********************************************************
WebMailHandler definition
***********************************************************/
function WebMailHandler(webMailNotifier){
  this.webMailNotifier=webMailNotifier;
}
WebMailHandler.prototype={
  loggedIn: false,
  logInToMail: true,
  charSet: null,
  data: "",
  mailCount:-1,
  onStartRequest : function (aRequest, aContext) {
    this.data = "";
  },
  onDataAvailable : function (aRequest, aContext, aStream, aSourceOffset, aLength){
    var is = Components.classes["@mozilla.org/intl/converter-input-stream;1"]
                     .createInstance(Components.interfaces.nsIConverterInputStream);
    var httpChannel=aContext.QueryInterface(Components.interfaces.nsIHttpChannel);
    is.init(aStream,
            this.charSet?this.charSet:(httpChannel.contentCharset?httpChannel.contentCharset:"utf-8"),
            1024,
            Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);
    var str = {};
    while (is.readString(4096, str) != 0) {
      this.data+=str.value;
    }
  },
  onStopRequest : function (aRequest, aContext, aStatus) {
    if(this.logInToMail){
      ////login result is mail page
      if(!this.loggedIn){
        this.loggedIn=true;
      }
      this.mailCount=this.getData(this.data);
      this.webMailNotifier.setState(this.id,this.mailCount);
    }else{
      if(!this.loggedIn){
        this.loggedIn=true;
        this.webMailNotifier.getHtml(this,this.getDataURL());
      }else{
        this.mailCount=this.getData(this.data);
        this.webMailNotifier.setState(this.id,this.mailCount);
      }
    }
  },
  checkMail : function(){
    if(!this.enabled)return;
    if(!this.loggedIn){
      this.webMailNotifier.getHtml(this,this.getLoginURL(),this.getLoginData());
    }else{
      this.webMailNotifier.getHtml(this,this.getDataURL());
    }
  },
  getUserID : function(){
    return this.webMailNotifier.prefBranch.getCharPref("webmail"+this.id+".userid");
  }
};
/***********************************************************
daum(hanmail)
***********************************************************/
function initDaumHandler(handler){
  handler.logInToMail=false;
  handler.charSet="euc-kr";
  handler.getLoginURL=function(){
      return "https://logins.daum.net/Mail-bin/login.cgi";
  };
  handler.getLoginData=function(){
    var userid=this.getUserID();
    return "url="+encodeURIComponent(this.getDataURL())
              +"&id="+userid+"&pw="+this.webMailNotifier.getPassword(this.id,userid);
  };
  handler.getDataURL=function(){
      return "http://mail.daum.net/hanmail/Welcome.daum?cid=&msg=";
  };
  handler.getMailURL=function(){
      return "http://mail.daum.net/hanmail/Index.daum";
  };
  handler.getData=function(aHtml){
    var fnd=aHtml.match(/\uc0c8\ud3b8\uc9c0[\s\S]+?>(\d+)<\/a>\ud1b5/);	//()
    if(fnd){
      return fnd[1];
    }else{
      this.loggedIn=false;
      return -1;
    }
  };
}
/***********************************************************
naver
***********************************************************/
function initNaverHandler(handler){
  handler.logInToMail=false;
  handler.getLoginURL=function(){
      return "https://nid.naver.com/nidlogin.login";
  };
  handler.getLoginData=function(){
    var userid=this.getUserID();
    return "url="+encodeURIComponent(this.getDataURL())
              +"&cert=no&nomap=0&saveID=0&ssllogin=1&id="
              +userid+"&pw="+this.webMailNotifier.getPassword(this.id,userid);
  };
  handler.getDataURL=function(){
      return "http://mail.naver.com";
  };
  handler.getMailURL=function(){
      return "http://mail.naver.com/list/?folder=0&m=1";
  };
  handler.getData=function(aHtml){
    var fnd=aHtml.match(/<a href([\s\S]{10,80}?)\uc548\uc77d\uc740\u0020\uba54\uc77c\uc774/);// 
    if(fnd){
      var fnd=fnd[1].match(/>(\d+)\uac1c/);//( )
      if(fnd){
        return fnd[1];
      }else return 0;
    }else{
      this.loggedIn=false;
      return -1;
    }
  };
}
/***********************************************************
empas
***********************************************************/
function initEmpasHandler(handler){
  handler.charSet="euc-kr";
  handler.getLoginURL=function(){
      return "https://login.empas.com/session/dologin.html";
  };
  handler.getLoginData=function(){
    var userid=this.getUserID();
    return "es="+encodeURIComponent(this.getDataURL())
              +"&userid="+userid+"&passwd="+this.webMailNotifier.getPassword(this.id,userid);
  };
  handler.getDataURL=function(){
      return "http://mail.empas.com/kr/main.html";
  };
  handler.getMailURL=function(){
      return "http://mail.empas.com/kr/mbox.html?n=1";
  };
  handler.getData=function(aHtml){
    var fnd=aHtml.match(/\uc548\u0020\uc77d\uc740\u0020\ud3b8\uc9c0[\s\S]*?(\d+)\ud1b5/);//   ( )
    if(fnd){
      return fnd[1];
    }else{
      this.loggedIn=false;
      return -1;
    }
  };
}

/***********************************************************
nate
***********************************************************/
function initNateHandler(handler){
  handler.logInToMail=false;
  handler.charSet="euc-kr";
  handler.getLoginURL=function(){
    return "https://xo.nate.com/servlets/LoginServlet";
  };
  handler.getLoginData=function(){
    var userid=this.getUserID();
    var ar=userid.split("@");
    return "redirect="+encodeURIComponent(this.getDataURL())
             +"&ID="+ar[0]+"&domain="+ar[1]
             +"&PASSWD="+this.webMailNotifier.getPassword(this.id,userid)+"&ssl=1";
  };
  handler.getDataURL=function(){
      return "http://mail.nate.com/mail_frame_left.html";
  };
  handler.getMailURL=function(){
      return "http://mail.nate.com/";
  };
  handler.getData=function(aHtml){
    var fnd=aHtml.match(/\uc804\uccb4\ubc1b\uc740\uba54\uc77c([\s\S]+?)<\/td>/);	//ü
    if(fnd){
      fnd=fnd[1].match(/\([\s\S]+?(\d+)[\s\S]+?\)/);
      if(fnd){
        return fnd[1];
      }else return 0;
    }
    this.loggedIn=false;
    return -1;
  };
}
/***********************************************************
Gmail
***********************************************************/
function initGmailHandler(handler){
  handler.logInToMail=false;
  handler.mailchecked=false;
  handler.getLoginURL=function(){
    return "https://www.google.com/accounts/ServiceLoginAuth";
  };
  handler.getLoginData=function(){
    var userid=this.getUserID();
    return "ltmpl=ca_tlsosm&ltmplcache=2&continue=http%3A%2F%2Fmail.google.com%2Fmail%2F%3F"
       +"&service=mail&rm=false&Email="+encodeURIComponent(userid)
       +"&Passwd="+this.webMailNotifier.getPassword(this.id,userid)+"&rmShown=1&null=Sign+in";
  };
  handler.getDataURL=function(){
    if(this.mailchecked){
      return "https://mail.google.com/mail/feed/atom";
    }else{
      this.mailchecked=true;
      var userid=this.getUserID();
      return "https://"+encodeURIComponent(userid)+":"+this.webMailNotifier.getPassword(this.id,userid)
                  +"@mail.google.com/mail/feed/atom";
    }
  };
  handler.getMailURL=function(){
      return "http://mail.google.com/";
  };
  handler.getData=function(aHtml){
    var fnd=aHtml.match(/<fullcount>(\d+)<\/fullcount>/);
    if(fnd){
      return fnd[1];
    }else{
      this.loggedIn=false;
      this.mailchecked=false;
      return -1;
    }
  };
}
/***********************************************************
Yahoo
***********************************************************/
function initYahooHandler(handler){
  handler.logInToMail=false;
  handler.charSet="euc-kr";
  handler.getLoginURL=function(){
    return "https://login.yahoo.com/config/login?";
  };
  handler.getLoginData=function(){
    var userid=this.getUserID();
    return "login="+userid+"&passwd="+this.webMailNotifier.getPassword(this.id,userid);
  };
  handler.getDataURL=function(){
      return "http://mail.yahoo.com/";
  };
  handler.getMailURL=function(){
      return "http://mail.yahoo.com/";
  };
  handler.getData=function(aHtml){
    var fnd=aHtml.match(/<div.+?folderlist.+?>([\s\S]+?)<\/div>\s*?<\/div>/);
    if(fnd){//original mode
      fnd=fnd[1].match(/<li .+?<\/a>/g);
      if(fnd){
        var s;
        var num=0;
        for each(s in fnd){
          var t=s.match(/id=\"(.+?)\"/);
          if(t){
            if(t[1]=="draft"||t[1]=="sent"
              ||t[1]=="bulk"||t[1]=="trash")continue;
          }
          s=s.match(/<\/div>.+?\((\d+)\)<\/a>/);
          if(s)num+=parseInt(s[1]);
        }
        return num;
      }
    }else{
      fnd=aHtml.match(/<ListFoldersResponse>([\s\S]+?)<\/ListFoldersResponse>/);
      if(fnd){
        fnd=fnd[1].match(/<folder.+?<\/folder>/g);
        if(fnd){
          var s;
          var num=0;
          for each(s in fnd){
            s=s.match(/unread=\"(\d+)\".+?fid=\"(.+?)\"/);
            if(s){
              if(s[2]=="%40B%40Bulk"||s[2]=="Draft"
                ||s[2]=="Sent"||s[2]=="Trash")continue;
              num+=parseInt(s[1]);
            }
          }
          return num;
        }
      }
    }
    this.loggedIn=false;
    return -1;
  };
}
/***********************************************************
Live(Hotmail)
***********************************************************/
function initLiveHandler(handler){
  handler.key=null;
  handler.getLoginURL=function(){
    return this.loginURL;
  };
  handler.getLoginData=function(){
    var userid=this.getUserID();
    return "login="+userid+"&passwd="+this.webMailNotifier.getPassword(this.id,userid)
              +"&PPFT="+this.key;
  };
  handler.getDataURL=function(){
      return "http://mail.live.com/mail/InboxLight.aspx";
  };
  handler.getMailURL=function(){
      return "http://mail.live.com/";
  };
  handler.getData=function(aHtml){
    var fnd=aHtml.match(/<ul.+?folderList.+?>([\s\S]+?)<\/ul>/);
    if(fnd){
      fnd=fnd[1].match(/InboxLight.aspx\?FolderID=.+?<\/a>/g);
      if(fnd){
        var s;
        var num=0;
        for each(s in fnd){
          s=s.match(/FolderID=.+?([^-]+?)&.+?\/>(.+?)<\/a>/);
          if(s){
            if(s[1]=="000000000005"||s[1]=="000000000004"
              ||s[1]=="000000000003"||s[1]=="000000000002")continue;
            s=s[2].match(/\((\d+)\)/);
            if(s)num+=parseInt(s[1]);
          }
        }
        return num;
      }
    }
    this.loggedIn=false;
    return -1;
  };
  handler.checkMail=function(){
    if(!this.enabled)return;
    if(!this.loggedIn){
      this.webMailNotifier.getHtml(this,"http://mail.live.com");
    }else{
      this.webMailNotifier.getHtml(this,this.getDataURL());
    }
  };
  handler.onStopRequest=function (aRequest, aContext, aStatus) {
    if(!this.key){
      var fnd=this.data.match(/srf_uPost=\'([\s\S]+?)\'/);
      if(fnd){
        this.loginURL=fnd[1];
        fnd=this.data.match(/PPFT[\s\S]+?value=\"(\S+?)\"/);
        if(fnd){
          this.key=fnd[1];
          this.webMailNotifier.getHtml(this,this.getLoginURL(),this.getLoginData());
        }
      }
      return;
    }
    if(!this.loggedIn){
      this.loggedIn=true;
      this.webMailNotifier.getHtml(this,this.getDataURL());
    }else{
      this.mailCount=this.getData(this.data);
      this.webMailNotifier.setState(this.id,this.mailCount);
    }
  }
}


